/**
 * 
 */
package eos.agents;

import java.util.Map;

import eos.datatypes.SmoothedValue;
import eos.framework.Economy;
import eos.framework.Good;
import eos.models.Laborer;
import static eos.models.Baseline3Model.*;

/**
 * @author Michael Adelson
 * 
 */
public class BL3Trader extends Baseline3Laborer {

    private static final double TARGET_FOOD_STOCK = 30.0,
            TARGET_MONEY_STOCK = 60.0;

    /**
     * @see Baseline3Firm#Baseline3Firm(Economy, Map)
     */
    public BL3Trader(Economy economy, Map<String, Good> goods) {
        super(economy, goods);
    }

    private SmoothedValue foodPrice = new SmoothedValue(
            getHistoricalPrice(this.food), 0.1);

    /*
     * (non-Javadoc)
     * 
     * @see eos.models.Baseline3Model.Baseline3Firm#baseLine3FirmSubclassAct()
     */
    protected void baseLine3LaborerSubclassAct() {
//        // buy food
//        if (this.foodMarket.getLastMarketPrice() <= 0.0
//                || this.laborMarket.getLastMarketPrice() <= 0.0)
//            this.foodMarket.addBuyOffer(this, Laborer.EAT_AMOUNT,
//                    HISTORICAL_FOOD_PRICE);
//        else {
//            final double quantum = Laborer.EAT_AMOUNT * 2.0; // Laborer.EAT_AMOUNT
//            // / 2.0;
//            double m = (this.foodMarket.getLastMarketPrice() - this.laborMarket
//                    .getLastMarketPrice())
//                    / TARGET_FOOD_STOCK;
//            double b = this.laborMarket.getLastMarketPrice();
//
//            if (m > 0.0)
//                this.foodMarket.addBuyOffer(this, Laborer.EAT_AMOUNT,
//                        this.foodMarket.getLastMarketPrice());
//
//            for (double x = this.food.getQuantity(); x <= 1.5 * TARGET_FOOD_STOCK; x += quantum) {
//                double price = m * x + b;
//                if (price <= 0.0)
//                    break;
//                this.foodMarket.addBuyOffer(this, quantum, price);
//            }
//        }
       
      if (this.foodMarket.getLastMarketPrice() <= 0.0
                || this.laborMarket.getLastMarketPrice() <= 0.0)
            this.foodMarket.addBuyOffer(this, Laborer.EAT_AMOUNT,
                    HISTORICAL_FOOD_PRICE);
        else{
          final double quantum = Laborer.EAT_AMOUNT * 1.0;
          double purse = this.money.getQuantity();
          for (double x = this.food.getQuantity(); x <= 1.5 * TARGET_FOOD_STOCK; x += quantum) {
            if(x == 0.0) x = .00000000001;
            double mod = .1*Math.pow(TARGET_FOOD_STOCK/x, .2) +.905;
            double price = mod*this.foodMarket.getLastMarketPrice();
            price = Math.min(price, purse);
            if (price <= 0.0)
              break;
            purse -= quantum*price;
            if(purse < 0.0) break;
            
            this.foodMarket.addBuyOffer(this, quantum, price);
          }
        }
      
        // sell labor
//        double price;
//        if (this.laborMarket.getLastMarketPrice() <= 0.0)
//            price = getHistoricalPrice(this.labor);
//        else {
//            final double frac = 0.95, range = 1.0 / frac - frac;
//            price = this.laborMarket.getLastMarketPrice()
//                    * (this.economy.random() * range + frac);
//        }
        // sell labor AD EDIT parabolic
        double price;
        if (this.laborMarket.getLastMarketPrice() <= 0.0)
            price = getHistoricalPrice(this.labor);
        else {
          if(this.laborMarket.getLastMarketPrice() > this.foodMarket.getLastMarketPrice() && this.food.getQuantity() < TARGET_FOOD_STOCK)
          {
            price = (this.laborMarket.getLastMarketPrice() - this.foodMarket.getLastMarketPrice()) * ((this.food.getQuantity()*this.food.getQuantity()) / (TARGET_FOOD_STOCK*TARGET_FOOD_STOCK)) + this.foodMarket.getLastMarketPrice();
          price = Math.min(price, this.laborMarket.getLastMarketPrice()*1.1);
          }
          else
            price = this.laborMarket.getLastMarketPrice();
          final double frac = 0.95, range = 1.0 / frac - frac;
            price *= /*this.laborMarket.getLastMarketPrice()
                    * */((Math.max(Math.min(this.economy.getRandom().nextGaussian(), 2.0),-2.0)/2)*(1 - frac) + 1);//(this.economy.random() * range + frac);
        }
        this.laborMarket.addSellOffer(this, Laborer.LABOR_ALLOWANCE, price);
        
        //buy utility
//        if (this.money.getQuantity() > TARGET_MONEY_STOCK
//                && this.food.getQuantity() > TARGET_FOOD_STOCK) {
//            // return;
//            double quantity = (this.money.getQuantity() - TARGET_MONEY_STOCK)
//                    / getHistoricalPrice(this.utility);
//            this.utilityMarket.addBuyOffer(this, quantity,
//                    getHistoricalPrice(this.utility));
//            //price *= 2.0; // raise price since the laborer doesn't NEED to work
//        }
        
         if (this.money.getQuantity() > TARGET_MONEY_STOCK
                && this.food.getQuantity() > TARGET_FOOD_STOCK) {
            // return;
          double uPrice;
        if (this.laborMarket.getLastMarketPrice() <= 0.0)
            uPrice = getHistoricalPrice(this.utility);
        else {
           uPrice = this.utilityMarket.getLastMarketPrice();
          final double frac = 0.95, range = 1.0 / frac - frac;
            uPrice *= ((Math.max(Math.min(this.economy.getRandom().nextGaussian(), 2.0),-2.0)/2)*(1 - frac) + 1);//(this.economy.random() * range + frac);
        }
        int quantity = (int)((this.money.getQuantity() - TARGET_MONEY_STOCK)/uPrice);
        if(quantity > 0)
            this.utilityMarket.addBuyOffer(this, quantity, uPrice);
         }

        //this.laborMarket.addSellOffer(this, Laborer.LABOR_ALLOWANCE, price);

        // trade food

        if (this.foodMarket.getLastMarketPrice() <= 0.0)
            return;

        this.foodPrice.update(this.foodMarket.getLastMarketPrice());

        if (this.economy.getRunningTimeSteps() < 200)
            return;

        double frac = 0.95;
        this.foodMarket.addBuyOffer(this, TARGET_FOOD_STOCK, this.foodPrice
                .getValue()
                * frac);

        double sellQuantity = this.food.getQuantity() - TARGET_FOOD_STOCK;
        if (sellQuantity > 0.0)
            this.foodMarket.addSellOffer(this, sellQuantity, this.foodPrice
                    .getValue()
                    / frac);

        // System.out
        // .println(this.foodPrice.getValue()
        // + " "
        // + this.foodMarket.getLastMarketPrice()
        // + " "
        // + sellQuantity
        // + " "
        // + (foodPrice.getValue() > this.foodMarket
        // .getLastMarketPrice() ? "greater" : "less"));

        // if (foodPrice.getValue() > this.foodMarket.getLastMarketPrice()) {
        // this.foodMarket.addBuyOffer(this, 2.0 * TARGET_FOOD_STOCK,
        // foodPrice.getValue());
        // } else if (this.food.getQuantity() - TARGET_FOOD_STOCK > 0.0) {
        // this.foodMarket.addSellOffer(this, this.food.getQuantity()
        // - TARGET_FOOD_STOCK, this.foodMarket.getLastMarketPrice());
        // }
    }
}
