/**
 * 
 */
package eos.agents;

import java.util.Map;

import eos.framework.Economy;
import eos.framework.Good;
import eos.models.Laborer;
import eos.utilities.Errors;
import static eos.models.Baseline4Model.*;

/**
 * An implementation for the Utility Factory in Baseline 4
 * 
 * @author Anthony DeLuise, Jr.
 * 
 */
public class BL4Factory extends Baseline4Firm {

    private static final double TARGET_UTILITY_STOCK = 60.0;
    
    /**
     * @see Baseline4Firm#Baseline4Firm(Economy, Map)
     */
    public BL4Factory(Economy economy, Map<String, Good> goods) {
        super(economy, goods);
        
        Errors.Assert(!this.isFarm);
    }

    /*
     * (non-Javadoc)
     * 
     * @see eos.models.Baseline4Model.Baseline4Firm#baseLine4FirmSubclassAct()
     */
    protected void baseLine4FirmSubclassAct() {
        // sell product 
         double productQuantity = this.product.getQuantity();
        if (this.productMarket.getLastMarketPrice() <= 0.0)
            this.productMarket.addSellOffer(this, this.product.getQuantity(),
                    HISTORICAL_UTILITY_PRICE);
        else {
            final double frac = 0.9, range = 1.0 / frac - frac;
            double quantum = this.product.getQuantity() / 10.0;

            while (this.product.getQuantity() > 0.0) {
                double price = this.productMarket.getLastMarketPrice()
                        * ((Math.max(Math.min(this.economy.getRandom().nextGaussian(), 2.0),-2.0)/2)*(1 - frac) + 1);
                this.productMarket.addSellOffer(this, quantum, price);
            }
        }
        

        // buy labor
        // initialize the market
        if (this.laborMarket.getLastMarketPrice() <= 0.0
                || this.productMarket.getLastMarketPrice() <= 0.0)
            this.laborMarket.addBuyOffer(this, Laborer.LABOR_ALLOWANCE,
                    getHistoricalPrice(this.labor));
        else { // use a demand curve
            double lastProduct = 0.0, minPrice = (Laborer.LABOR_ALLOWANCE * this.laborMarket
                    .getLastMarketPrice()) / 2.0;
            for (double d = Laborer.LABOR_ALLOWANCE; this.money.getQuantity() > 0.0; d += Laborer.LABOR_ALLOWANCE) {
                double marginalProduct = this.convertToProduct(d) - lastProduct;
                
                if (lastProduct + productQuantity >= TARGET_UTILITY_STOCK)
                    break;

                double price = marginalProduct
                        * this.productMarket.getLastMarketPrice();

                // stop when you'd be buying at < 1/2 the price of labor
                if (price < minPrice)
                    break;

                this.laborMarket.addBuyOffer(this, Laborer.LABOR_ALLOWANCE,
                        price);

                lastProduct += marginalProduct;
            }
        }
    }
}
