/**
 * 
 */
package eos.agents;

import java.util.Map;

import eos.framework.Economy;
import eos.framework.Good;
import eos.models.Laborer;
import eos.utilities.Errors;
import static eos.models.Baseline4Model.*;

/**
 * A farm implementation for the Baseline 4 economy
 * 
 * @author Anthony DeLuise, Jr.
 * 
 */
public class BL4Farm extends Baseline4Firm {

    private static double TARGET_FOOD_STOCK = 120.0;
    private static double TARGET_OWNER_FOOD_STOCK = 30.0;
    private static double RANDOM_FACTOR = .01;
    

    /**
     * @see Baseline4Firm#Baseline4Firm(Economy, Map)
     */
    public BL4Farm(Economy economy, Map<String, Good> goods) {
        super(economy, goods);

        Errors.Assert(this.isFarm);
    }

    /*
     * (non-Javadoc)
     * 
     * @see eos.models.Baseline4Model.Baseline4Firm#baseLine4FirmSubclassAct()
     */
    protected void baseLine4FirmSubclassAct() {
        // sell food
        double productQuantity = this.product.getQuantity();
        if(productQuantity == 0.0) return;
        if (this.productMarket.getLastMarketPrice() <= 0.0)
            this.productMarket.addSellOffer(this, this.product.getQuantity(),
                    HISTORICAL_FOOD_PRICE);
        else {
            final double frac = 0.9, range = 1.0 / frac - frac;
            double quantum = this.product.getQuantity() / 10.0;

            while (this.product.getQuantity() > 0.0) {
                double price = this.productMarket.getLastMarketPrice()
                        * ((Math.max(Math.min(this.economy.getRandom().nextGaussian(), 2.0),-2.0)/2)*(1 - frac) + 1);
                this.productMarket.addSellOffer(this, quantum, price);
            }
        }

        // buy labor

        double ownerFoodStock = this.getOwner().asAgent().getGood("Food").getQuantity();
        if( ownerFoodStock < .75*TARGET_OWNER_FOOD_STOCK && this.product.getQuantity() < TARGET_FOOD_STOCK){
        final double quantum = Laborer.LABOR_ALLOWANCE * 1.0;
          double purse = this.money.getQuantity();
          for (double x = ownerFoodStock; x <= 1.5 * TARGET_OWNER_FOOD_STOCK; x += quantum) {
            if(x == 0.0) x = .00000000001;
            double mod = .1*Math.pow(TARGET_OWNER_FOOD_STOCK/x, .2) +.90/*5*/+this.economy.random()*RANDOM_FACTOR;
            double price = mod*this.laborMarket.getLastMarketPrice();
            price = Math.min(price, purse);
            if (price <= 0.0)
              break;
            purse -= quantum*price;
            if(purse < 0.0) break;
            
            this.laborMarket.addBuyOffer(this, quantum, price);
          }
        }

        
// initialize the market
        if (this.laborMarket.getLastMarketPrice() <= 0.0
                || this.productMarket.getLastMarketPrice() <= 0.0)
            this.laborMarket.addBuyOffer(this, Laborer.LABOR_ALLOWANCE,
                    getHistoricalPrice(this.labor));
        else { // use a demand curve
            double lastProduct = 0.0, minPrice = (Laborer.LABOR_ALLOWANCE * this.laborMarket
                    .getLastMarketPrice()) / 2.0;
            for (double d = Laborer.LABOR_ALLOWANCE; this.money.getQuantity() > 0.0; d += Laborer.LABOR_ALLOWANCE) {
                double marginalProduct = this.convertToProduct(d) - lastProduct;

                if (lastProduct + productQuantity >= TARGET_FOOD_STOCK)
                    break;

                double price = marginalProduct
                        * this.productMarket.getLastMarketPrice();

                // stop when you'd be buying at < 1/2 the price of labor
                if (price < minPrice)
                    break;

                this.laborMarket.addBuyOffer(this, Laborer.LABOR_ALLOWANCE,
                        price);

                lastProduct += marginalProduct;
            }
        }
    }
}
