/**
 * 
 */
package eos.agents;

import java.util.Map;
import java.lang.Math;

import eos.framework.*;
import eos.models.Laborer;
import static eos.models.Baseline4Model.*;

/**
 * A laborer implementation for the baseline 4 economy.
 * 
 * @author Anthony DeLuise, Jr.
 * 
 */
public class BL4Laborer extends Baseline4Laborer {

    private static final double TARGET_FOOD_STOCK = 30.0,
            TARGET_MONEY_STOCK = 60.0,
      RANDOM_FACTOR = .01;

    /**
     * @see Baseline4Laborer#Baseline4Laborer(Economy, Map)
     */
    public BL4Laborer(Economy economy, Map<String, Good> goods) {
        super(economy, goods);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * eos.models.Baseline4Model.Baseline4Laborer#baseLine4LaborerSubclassAct()
     */
    protected void baseLine4LaborerSubclassAct() {
        // buy food
      
        if (this.foodMarket.getLastMarketPrice() <= 0.0
                || this.laborMarket.getLastMarketPrice() <= 0.0)
            this.foodMarket.addBuyOffer(this, Laborer.EAT_AMOUNT,
                    HISTORICAL_FOOD_PRICE);
        else{
          final double quantum = Laborer.EAT_AMOUNT * 1.0;
          double purse = this.money.getQuantity();
          for (double x = this.food.getQuantity(); x <= 1.5 * TARGET_FOOD_STOCK; x += quantum) {
            if(x == 0.0) x = .00000000001;
            double mod = .1*Math.pow(TARGET_FOOD_STOCK/x, .2) +.90/*5*/+this.economy.random()*RANDOM_FACTOR;
            double price = mod*this.foodMarket.getLastMarketPrice();
            price = Math.min(price, purse);
            if (price <= 0.0)
              break;
            purse -= quantum*price;
            if(purse < 0.0) break;
            
            this.foodMarket.addBuyOffer(this, quantum, price);
          }
        }
          
        
        // sell food
        
        if(this.food.getQuantity() > 1.5*TARGET_FOOD_STOCK)
        {
          
          final double quantum = Laborer.EAT_AMOUNT * 1.0;
          for (double x = this.food.getQuantity(); x > 1.5*TARGET_FOOD_STOCK; x -= quantum) {
            if(x == 0.0) x = .00000000001;
            double mod = .1*Math.pow(TARGET_FOOD_STOCK/(x - TARGET_FOOD_STOCK), .2) +.90/*5*/+this.economy.random()*RANDOM_FACTOR;
            double price = mod*this.foodMarket.getLastMarketPrice();
            if (price <= 0.0)
              break;            
            this.foodMarket.addSellOffer(this, quantum, price);
          }
          
        }
        
        // sell labor 
        double price;
        if (this.laborMarket.getLastMarketPrice() <= 0.0)
            price = getHistoricalPrice(this.labor);
        else {
          if(this.laborMarket.getLastMarketPrice() > this.foodMarket.getLastMarketPrice() && this.food.getQuantity() < TARGET_FOOD_STOCK)
          {
            price = (this.laborMarket.getLastMarketPrice() - this.foodMarket.getLastMarketPrice()) * ((this.food.getQuantity()*this.food.getQuantity()) / (TARGET_FOOD_STOCK*TARGET_FOOD_STOCK)) + this.foodMarket.getLastMarketPrice();
          price = Math.min(price, this.laborMarket.getLastMarketPrice()*1.1);
          }
          else
            price = this.laborMarket.getLastMarketPrice();
          final double frac = 0.95, range = 1.0 / frac - frac;
            price *=((Math.max(Math.min(this.economy.getRandom().nextGaussian(), 2.0),-2.0)/2)*(1 - frac) + 1);//(this.economy.random() * range + frac);
        }
        this.laborMarket.addSellOffer(this, Laborer.LABOR_ALLOWANCE, price);

        
        //buy utility
        if (this.money.getQuantity() > TARGET_MONEY_STOCK
                && this.food.getQuantity() > TARGET_FOOD_STOCK) {
          double uPrice;
        if (this.laborMarket.getLastMarketPrice() <= 0.0)
            uPrice = getHistoricalPrice(this.utility);
        else {
           uPrice = this.utilityMarket.getLastMarketPrice();
          final double frac = 0.95, range = 1.0 / frac - frac;
            uPrice *= ((Math.max(Math.min(this.economy.getRandom().nextGaussian(), 2.0),-2.0)/2)*(1 - frac) + 1);
        }
        int quantity = (int)((this.money.getQuantity() - TARGET_MONEY_STOCK)/uPrice);
        if(quantity > 0 && uPrice > 0.0)
            this.utilityMarket.addBuyOffer(this, quantity, uPrice);
        }

        
    }
}
