/**
 * 
 */
package eos.agents;

import java.util.Map;

import eos.framework.Economy;
import eos.framework.Good;
import eos.models.Laborer;
import static eos.models.Baseline4Model.*;

/**
 * An owner implementation for the baseline 4 economy. 
 * 
 * @author Anthony DeLuise, Jr.
 * 
 */
public class BL4Owner extends Baseline4Laborer {

    /**
     * Target values used to assess need and make bidding decisions.
     */
    private static final double TARGET_FOOD_STOCK = 30.0,
            TARGET_MONEY_STOCK = 60.0, TARGET_FIRM_MONEY_STOCK = 100.0, RANDOM_FACTOR = .01;

    /**
     * @see Baseline4Laborer#Baseline4Laborer(Economy, Map)
     */
    public BL4Owner(Economy economy, Map<String, Good> goods) {
        super(economy, goods);
        // TODO Auto-generated constructor stub
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * eos.models.Baseline4Model.Baseline4Laborer#baseLine4LaborerSubclassAct()
     */
    protected void baseLine4LaborerSubclassAct() {
        // withdraw money
        final Good firmMoney = this.firms.get(0).asAgent().getGood(
                this.money.getName());
        double moneyToWithdraw = Math.max(TARGET_MONEY_STOCK
                - this.money.getQuantity(), firmMoney.getQuantity()
                - TARGET_FIRM_MONEY_STOCK);
        if (moneyToWithdraw > 0.0)
            this.firms.get(0).requestTransfer(this.money.getName(),
                    moneyToWithdraw);

        // buy food 
        if (this.foodMarket.getLastMarketPrice() <= 0.0
              || this.laborMarket.getLastMarketPrice() <= 0.0)
          this.foodMarket.addBuyOffer(this, Laborer.EAT_AMOUNT,
                                      HISTORICAL_FOOD_PRICE);
        else{
          final double quantum = Laborer.EAT_AMOUNT * 1.0;
          double purse = this.money.getQuantity();
          for (double x = this.food.getQuantity(); x <= 1.5 * TARGET_FOOD_STOCK; x += quantum) {
            if(x == 0.0) x = .00000000001;
            double mod = .1*Math.pow(TARGET_FOOD_STOCK/x, .2) +.90/*5*/+this.economy.random()*RANDOM_FACTOR;
            double price = mod*this.foodMarket.getLastMarketPrice();
            price = Math.min(price, purse);
                if (price <= 0.0)
                  break;
                purse -= quantum*price;
                if(purse < 0.0) break;
                
                this.foodMarket.addBuyOffer(this, quantum, price);
          }
        }
        
        //Sell Food
        if(this.food.getQuantity() > 1.5*TARGET_FOOD_STOCK)
        {          
           final double quantum = Laborer.EAT_AMOUNT * 1.0;
          for (double x = this.food.getQuantity(); x > 1.5*TARGET_FOOD_STOCK; x -= quantum) {
            if(x == 0.0) x = .00000000001;
            double mod = .1*Math.pow(TARGET_FOOD_STOCK/(x - TARGET_FOOD_STOCK), .2) +.90/*5*/+this.economy.random()*RANDOM_FACTOR;
            double price = mod*this.foodMarket.getLastMarketPrice();
            if (price <= 0.0)
              break;            
            this.foodMarket.addSellOffer(this, quantum, price);
          }
        }
        
        // buy utility
        if (this.money.getQuantity() > TARGET_MONEY_STOCK
                && this.food.getQuantity() > TARGET_FOOD_STOCK) {
        double uPrice;
        if (this.utilityMarket.getLastMarketPrice() <= 0.0)
            uPrice = getHistoricalPrice(this.utility);
        else {
           uPrice = this.utilityMarket.getLastMarketPrice();
          final double frac = 0.95, range = 1.0 / frac - frac;
            uPrice *= ((Math.max(Math.min(this.economy.getRandom().nextGaussian(), 2.0),-2.0)/2)*(1 - frac) + 1);//(this.economy.random() * range + frac);
        }
        int quantity = (int)((this.money.getQuantity() - TARGET_MONEY_STOCK)/uPrice);
        if(quantity > 0 && uPrice > 0.0)
            this.utilityMarket.addBuyOffer(this, quantity, uPrice);
        }

    }

}
