/**
 * 
 */
package eos.io;

import eos.utilities.Errors;

/**
 * Like CSVMultiRunPrinter, but prints only the cleanup data from each "run" and
 * places data from each run in a single column. Automatically generates a Run
 * column.
 * 
 * @author Michael Adelson
 */
public class CSVMultiRunCleanupPrinter extends CSVPrinter implements
        MultiRunPrinter {

    /**
     * The current run number.
     */
    private int runCount = 0;

    /**
     * Construct a CSVMultiRunCleanupPrinter for fileName. Warns if period is
     * non-zero.
     */
    public CSVMultiRunCleanupPrinter(String fileName, int period) {
        this(fileName);

        Errors.warnIf(period != 0, this.getClass().getSimpleName()
                + " ignoring non-zero period " + period);
    }

    /**
     * Construct a CSVMultiRunCleanupPrinter.
     */
    public CSVMultiRunCleanupPrinter(String fileName) {
        super(fileName, 0);
    }

    /**
     * Close the CSVMultiRunCleanupPrinter.
     */
    public void cleanup() {
        this.cleanup(null);
    }

    /**
     * This is the only time CSVMultiRunCleanupPrinter prints data. To close the
     * MultiRunPrinter, call cleanup with null or call cleanup().
     */
    public void cleanup(DataOutputEconomy economy) {
        if (economy == null) {
            this.printWriter.close();
            return;
        }

        StringBuilder sb = new StringBuilder();

        this.runCount++;
        sb.append(this.runCount + CS);

        for (DataPrinter dataPrinter : dataPrinters) {
            sb.append(dataPrinter.getCleanupData(economy));
            sb.append(CS);
        }

        printWriter.println(sb.toString());
    }

    /**
     * Does nothing.
     */
    public void print(DataOutputEconomy economy) {
        return;
    }

    /**
     * Prints titles.
     */
    public void printTitles(DataOutputEconomy economy) {
        if (this.runCount == 0) {
            this.printWriter.print("Run" + CS);
            super.printTitles(economy);
        }
    }
}
