/**
 * 
 */
package eos.io;

import java.util.*;

/**
 * This DataPrinter can be used to generate fixed-data columns. For example, it
 * can be used to add a description of each run when using a
 * CSVMultiRunCleanupPrinter.
 * 
 * @author Michael Adelson
 */
public class FixedDataDataPrinter implements DataPrinter {

    // a map of economies to data strings.
    protected Map<DataOutputEconomy, String> map;

    // the column name under which the data is printed.
    private final String columnName;

    /**
     * Construct a new FixedDataDataPrinter whose data will print under
     * columnName.
     */
    public FixedDataDataPrinter(String columnName) {
        this.map = new HashMap<DataOutputEconomy, String>();
        this.columnName = columnName;
    }
    
    /**
     * Set data to be printed for this economy.
     */
    public void setData(DataOutputEconomy economy, String data) {
        this.map.put(economy, data);
    }

    /**
     * Calls this.getData() and removes the mapping for economy to free it for
     * garbage collection.
     */
    public String getCleanupData(DataOutputEconomy economy) {
        String data = this.getData(economy);
        this.map.remove(economy);

        return data;
    }

    /**
     * Prints data for economy set by setData(). Prints "" if no data was set.
     */
    public String getData(DataOutputEconomy economy) {
        String data = this.map.get(economy);

        return (data != null) ? data : "";
    }

    /**
     * Get the title set by the constructor. Default is "Notes."
     */
    public String getTitle(DataOutputEconomy economy) {
        return (this.columnName != null) ? this.columnName : "Notes";
    }

    /**
     * True.
     */
    public boolean isCSVCompatible() {
        return true;
    }

    /**
     * True.
     */
    public boolean isMultiRunCompatible() {
        return true;
    }

}
