/**
 * 
 */
package eos.simulations;

import java.util.*;
import eos.agents.*;
import eos.economies.BL321Economy;
import eos.framework.Economy;
import eos.io.GoodTrackerDataPrinter;
import eos.io.Simulation;
import eos.io.Simulation.InitializationFunction;
import eos.markets.*;
import eos.models.*;
import eos.models.Baseline3Model.*;
/**
 * @author madelson
 *
 */
public class BL3Sim2 {
    
    // parameters
    static int numFarms = 50;
    static int numLaborers = 36 * numFarms;
    static int numBLaborers = 0; //10;
    
    static Simulation sim;
    
    static {
        sim = new Simulation("BL3-2", BL321Economy.class, /*678294307504387L*/new Random().nextLong());
        sim.addGoods(Baseline3Good.class, "Food", "Labor", "Money", "Utility");
        
        //sim.addMarkets(CallAuctionMarket.class, "Money", "Food", "Labor", "Utility");
        sim.addMarkets(BuyerPostedPriceBargainingMarket.class, "Money", "Labor");
        sim.addMarkets(CallAuctionMarket.class, "Money", "Food", "Utility");
        
        sim.addAgents(BL3Laborer.class, numLaborers - numBLaborers);
        sim.addAgents(BL3BudgetLaborer.class, numBLaborers);
        //sim.addAgents(BL3Trader.class, 1);
        sim.addOwnersAndFirms(BL3Owner.class, numFarms, BL3Farm.class, 1);
        sim.addOwnersAndFirms(BL3Owner.class, numFarms, BL3Factory.class, 1);
        
        sim.setQuantities(BL3Laborer.class, 33, 0, 60, 0);
        sim.setQuantities(BL3BudgetLaborer.class, 33, 0, 60, 0);
        sim.setQuantities(BL3Owner.class, 33, 0, 90, 0);
        sim.setQuantities(BL3Farm.class, 20, 0, 100, 0);
        sim.setQuantities(BL3Factory.class, 0, 0, 100, 5);
        //sim.setQuantities(BL3Trader.class, 400, 0, 100, 0);
        
        // set the prices before running
        sim.addInitializationFunction(new InitializationFunction() {
            public void initialize(Economy economy) {
                Baseline3Model.HISTORICAL_LABOR_PRICE = 0.492;
                Baseline3Model.HISTORICAL_FOOD_PRICE = .2;
                //Baseline3Model.HISTORICAL_UTILITY_PRICE = 0.2;
            }
        });
    }
    
    /**
     * @param args
     */
    public static void main(String[] args) {
        //sim.setDefaultPrintPeriod(1);
        sim.setPerturbation(10);
        sim.addDataPrinter(new GoodTrackerDataPrinter("Money", false));
        long clock = System.currentTimeMillis();
        sim.run(20000);
        System.out.println(System.currentTimeMillis() - clock);
    }

}
