package eos.utilities;


/**
 * A class to provide general-purpose string-processing methods.
 * 
 * By Michael Adelson
 */
public class Strings {

    /**
     * Is the string Null or Empty?
     */
    public static boolean isNoE(String s) {
        if (s == null)
            return true;
        if (s.length() == 0)
            return true;
        return false;
    }

    /**
     * Returns a single class name or a list of class names. 
     */
    public static String getShortNames(Class<?>... classes) {
        String s = "";
        if (classes == null || classes.length == 0)
            return s;

        for (Class<?> cls : classes)
            s += cls.getName().substring(cls.getName().lastIndexOf('.') + 1)
                    + ", ";
        
        return s.substring(0, s.length() - 2); // strip off last commas
    }

    /**
     * Return the String containing num and its suffix: e.g. "1st", "2nd",
     * "4th".
     */
    public static String getOrdinal(int num) {
        int n = Math.abs(num);
        while (n / 10 > 0)
            n /= 10;
        switch (n) {
        case 1:
            return num + "st";
        case 2:
            return num + "nd";
        case 3:
            return num + "rd";
        default:
            return num + "th";
        }
    }

    /**
     * @return A capitalized version of s
     */
    public static String capitalize(String s) {
        if (s.length() == 0)
            return s;
        return s.substring(0, 1).toUpperCase() + s.substring(1).toLowerCase();
    }
}
